/* tray.TIcon.java
* TrayIcon Library for Processing
* needs Java 1.6
* created by derDoc (derDoc@gmx.de)
*/


package tray;

import java.awt.*;
//import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

import processing.core.*;


public class TIcon extends PGraphics3D {


	ColorModel _cm;
  	MemoryImageSource _mis;
  	TrayIcon ti;
  	SystemTray tray;
	
	public static final String TRAY = "tray.TIcon";


	public void setParent(PApplet p) {
    	super.setParent(p);
		//if not supported, or not right Java version, exit
		if ( (p != null && p.javaVersion < 1.6f) || (!SystemTray.isSupported()) ){
			throw new RuntimeException("SystemTray not supported");
		}

  	}

	
	//dont call
	public TIcon() { }
	
	/*
	public TIcon(int w, int h) {
		this(w,h,null);
	} 
	
	public TIcon(int w, int h, PApplet p) {
		super(w,h,p);
	} 
	*/
	
//	----------------------------------------- P5 functions
	
	protected void allocate(){
	
		//call super to set up everything
		super.allocate();
		
		//called to allocate and set up buffers
		//use here to make image and add it to the System Statusbar
	
        tray = SystemTray.getSystemTray();
		//System.out.println("In TIcon.allocate(), systray is: "+tray);
		 		 
         //if not main surface create the image ourselves, since its not accesible outside class (its private in PGraphics3D) prepend underscore '_'
         if (!this.primarySurface) {
         	//System.out.println("In TIcon.allocate(), not main drawing surf.");
		
         	_cm = new DirectColorModel(32, 0x00ff0000, 0x0000ff00, 0x000000ff);
      		//_mis = new MemoryImageSource(width, height, pixels, 0, width);
      		_mis = new MemoryImageSource(width, height, _cm, pixels, 0, width);
      		_mis.setFullBufferUpdates(true);
      		_mis.setAnimated(true);
      		image = Toolkit.getDefaultToolkit().createImage(_mis);
      		//System.out.println("In TIcon.allocate(), created Image: "+image);

       	 }  
		 //and make the TIcon out of the Image used for drawing
		 
         ti = new TrayIcon(image, "Processing Sketch");
         //System.out.println("In TIcon.allocate(), Img size is: " + image.getHeight(null) + "," + image.getWidth(null));
      	 //System.out.println("In TIcon.allocate(), created TrayIcon: "+ti);
      	 ti.setImageAutoSize(true);
         
         try {
             tray.add(ti);
      	 	 //System.out.println("Added Icon to tray, its size is:"+ti.getSize().toString());
         } catch (AWTException e) {
             System.err.println("Error adding Icon to tray: " + e);
         }
	}
	
	
	public void dispose() {		//do some cleanup
  		tray.remove(ti);
  		ti.setImage(null);
  		ti = null;
  	}


  	public boolean displayable() {
    	return false;  // just in case someone wants to use this as the main drawing surface
  	}
	
	
	
	public void endDraw(){
	
		super.endDraw();
		//and let our own mis know that there are new pixels
		if (!this.primarySurface) {
			//_mis.newPixels(pixels, _cm, 0, width);
			_mis.newPixels(0, 0, width, height, true);
		}
		//reset the new image into the tray icon
		if (ti != null)
			ti.setImage(image);
	}


	
//	----------------------------------------- own functions

	
	public void setTooltip(String tip){
		ti.setToolTip(tip);
	}
	
	public void setMenu(PopupMenu m){
		ti.setPopupMenu(m);
	}
	
// -------------------------- run from jar functions

	public static void main(String args[]){
		new TIcon(500,500);
	}
	
	//this creates a new dock instance, when started directly from the jar, cant use the empty _ctor here because it gets called from P5 as well
	//that messes up the init'ing from within processing, preventing it from working correctly (black image)
	public TIcon(int w, int h) {
		this.setSize(w,h);
		initFromJar(w,h);
	}
	
	private void initFromJar(int w, int h){
	
		ImageIcon imgic;
		JLabel jl;
	
		//create the frame
		Frame f = new Frame("TIcon test");
		f.setSize(w,h);
		//f.pack();
		f.setVisible(true);
		
		//just load some image here, just put in the dir and change names in the following 2 lines
		//both are from the P5 dist, BTW
		Image c = Toolkit.getDefaultToolkit().getImage("icon.gif");
		Image b = Toolkit.getDefaultToolkit().getImage("loading.gif");
		
		while (b.getWidth(null) < 0 && b.getHeight(null) < 0 ){
			try{ Thread.sleep(10); } catch(Exception e) {}
		}
		while (c.getWidth(null) < 0 && c.getHeight(null) < 0 ){
			try{ Thread.sleep(10); } catch(Exception e) {}
		}
		
		tray = SystemTray.getSystemTray();
        
        ti = new TrayIcon(b, "Processing Sketch");
         System.out.println("In TIcon.allocate(), Img size is: " + image.getHeight(null) + "," + image.getWidth(null));
      	 System.out.println("In TIcon.allocate(), created TrayIcon: "+ti);
      	 ti.setImageAutoSize(true);
         
         try {
             tray.add(ti);
      	 	 System.out.println("Added Icon to tray, its size is:"+ti.getSize().toString());
         } catch (AWTException e) {
             System.err.println("Error adding Icon to tray: " + e);
         }
			
		imgic = new ImageIcon(b);
		jl = new JLabel(imgic);
		f.add(jl);

		f.pack();
		f.setVisible(true);

		try{ Thread.sleep(5000); } catch(Exception e) { e.printStackTrace(); }
		
		 //ti = new TrayIcon(c, "Processing Sketch");
         //System.out.println("In TIcon.allocate(), Img size is: " + image.getHeight(null) + "," + image.getWidth(null));
      	 //System.out.println("In TIcon.allocate(), created TrayIcon: "+ti);
      	 
      	 imgic = new ImageIcon(c);
		 jl = new JLabel(imgic);
		 f.add(jl);
		 f.pack();
		 f.setVisible(true);

         ti.setImage(c);

		
//		try{ Thread.sleep(1000); } catch(Exception e) { e.printStackTrace(); }
		
//		System.exit(0);
	}

	
}








